@@ -6,7 +6,7 @@ use tendermint_proto::Protobuf;
 
 use ibc_proto::ibc::apps::transfer::v1::MsgTransfer as RawMsgTransfer;
 
-use crate::application::ics20_fungible_token_transfer::error::{Error, Kind};
+use crate::application::ics20_fungible_token_transfer::error::Error;
 use crate::ics02_client::height::Height;
 use crate::ics24_host::identifier::{ChannelId, PortId};
 use crate::signer::Signer;
@@ -54,16 +54,16 @@ impl Msg for MsgTransfer {
 impl Protobuf<RawMsgTransfer> for MsgTransfer {}
 
 impl TryFrom<RawMsgTransfer> for MsgTransfer {
-    type Error = Kind;
+    type Error = Error;
 
     fn try_from(raw_msg: RawMsgTransfer) -> Result<Self, Self::Error> {
         let timeout_timestamp = Timestamp::from_nanoseconds(raw_msg.timeout_timestamp)
-            .map_err(|_| Kind::InvalidPacketTimeoutTimestamp(raw_msg.timeout_timestamp))?;
+            .map_err(|_| Error::invalid_packet_timeout_timestamp(raw_msg.timeout_timestamp))?;
 
         let timeout_height = match raw_msg.timeout_height.clone() {
             None => Height::zero(),
             Some(raw_height) => raw_height.try_into().map_err(|e| {
-                Kind::InvalidPacketTimeoutHeight(format!("invalid timeout height {}", e))
+                Error::invalid_packet_timeout_height(format!("invalid timeout height {}", e))
             })?,
         };
 
@@ -71,11 +71,11 @@ impl TryFrom<RawMsgTransfer> for MsgTransfer {
             source_port: raw_msg
                 .source_port
                 .parse()
-                .map_err(|_| Kind::InvalidPortId(raw_msg.source_port.clone()))?,
+                .map_err(|e| Error::invalid_port_id(raw_msg.source_port.clone(), e))?,
             source_channel: raw_msg
                 .source_channel
                 .parse()
-                .map_err(|_| Kind::InvalidChannelId(raw_msg.source_channel.clone()))?,
+                .map_err(|e| Error::invalid_channel_id(raw_msg.source_channel.clone(), e))?,
             token: raw_msg.token,
             sender: raw_msg.sender.into(),
             receiver: raw_msg.receiver.into(),
