@@ -4,7 +4,7 @@ use tendermint_proto::Protobuf;
 
 use ibc_proto::ibc::core::client::v1::MsgSubmitMisbehaviour as RawMsgSubmitMisbehaviour;
 
-use crate::ics02_client::error::{Error, Kind};
+use crate::ics02_client::error::Error;
 use crate::ics02_client::misbehaviour::AnyMisbehaviour;
 use crate::ics24_host::identifier::ClientId;
 use crate::signer::Signer;
@@ -42,13 +42,15 @@ impl TryFrom<RawMsgSubmitMisbehaviour> for MsgSubmitAnyMisbehaviour {
     type Error = Error;
 
     fn try_from(raw: RawMsgSubmitMisbehaviour) -> Result<Self, Self::Error> {
-        let raw_misbehaviour = raw.misbehaviour.ok_or(Kind::InvalidRawMisbehaviour)?;
+        let raw_misbehaviour = raw
+            .misbehaviour
+            .ok_or_else(Error::missing_raw_misbehaviour)?;
 
         Ok(MsgSubmitAnyMisbehaviour {
             client_id: raw
                 .client_id
                 .parse()
-                .map_err(|e| Kind::InvalidRawMisbehaviour.context(e))?,
+                .map_err(Error::invalid_raw_misbehaviour)?,
             misbehaviour: AnyMisbehaviour::try_from(raw_misbehaviour)?,
             signer: raw.signer.into(),
         })
