@@ -3,7 +3,7 @@ use std::convert::TryFrom;
 use ibc_proto::ibc::core::connection::v1::Version as RawVersion;
 use tendermint_proto::Protobuf;
 
-use crate::ics04_channel::error::{Error, Kind};
+use crate::ics04_channel::error::Error;
 use std::str::FromStr;
 
 #[derive(Clone, Debug, PartialEq, Eq)]
@@ -17,7 +17,7 @@ pub struct Version {
 impl Protobuf<RawVersion> for Version {}
 
 impl TryFrom<RawVersion> for Version {
-    type Error = anomaly::Error<Kind>;
+    type Error = Error;
     fn try_from(value: RawVersion) -> Result<Self, Self::Error> {
         Ok(Version {
             identifier: value.identifier,
@@ -48,7 +48,7 @@ impl FromStr for Version {
     type Err = Error;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        Ok(Version::decode(s.as_bytes()).map_err(|e| Kind::InvalidVersion.context(e))?)
+        Version::decode(s.as_bytes()).map_err(Error::invalid_version)
     }
 }
 
@@ -76,8 +76,7 @@ pub fn pick_version(
 ) -> Result<String, Error> {
     let mut intersection: Vec<Version> = vec![];
     for s in supported_versions.iter() {
-        let supported_version =
-            Version::decode(s.as_bytes()).map_err(|e| Kind::InvalidVersion.context(e))?;
+        let supported_version = Version::decode(s.as_bytes()).map_err(Error::invalid_version)?;
         for c in counterparty_versions.iter() {
             let counterparty_version = Version::from_str(c.as_str())?;
             if supported_version.identifier != counterparty_version.identifier {
@@ -89,16 +88,14 @@ pub fn pick_version(
     }
     intersection.sort_by(|a, b| a.identifier.cmp(&b.identifier));
     if intersection.is_empty() {
-        return Err(Kind::NoCommonVersion.into());
+        return Err(Error::no_common_version());
     }
     Ok(intersection[0].to_string())
 }
 
 pub fn validate_versions(versions: Vec<String>) -> Result<Vec<String>, Error> {
     if versions.is_empty() {
-        return Err(Kind::InvalidVersion
-            .context("no versions".to_string())
-            .into());
+        return Err(Error::empty_version());
     }
     for version_str in versions.iter() {
         validate_version(version_str.clone())?;
@@ -107,19 +104,14 @@ pub fn validate_versions(versions: Vec<String>) -> Result<Vec<String>, Error> {
 }
 
 pub fn validate_version(raw_version: String) -> Result<String, Error> {
-    let version =
-        Version::from_str(raw_version.as_ref()).map_err(|e| Kind::InvalidVersion.context(e))?;
+    let version = Version::from_str(raw_version.as_ref())?;
 
     if version.identifier.trim().is_empty() {
-        return Err(Kind::InvalidVersion
-            .context("empty version string".to_string())
-            .into());
+        return Err(Error::empty_version());
     }
     for feature in version.features {
         if feature.trim().is_empty() {
-            return Err(Kind::InvalidVersion
-                .context("empty feature string".to_string())
-                .into());
+            return Err(Error::empty_version());
         }
     }
     Ok(raw_version)
