@@ -3,16 +3,14 @@ use std::str::FromStr;
 use std::time::Duration;
 
 use serde::{Deserialize, Serialize};
-use tendermint::trust_threshold::{
-    TrustThresholdFraction as TrustThreshold, TrustThresholdFraction,
-};
 use tendermint_proto::Protobuf;
 
-use ibc_proto::ibc::lightclients::tendermint::v1::{ClientState as RawClientState, Fraction};
+use ibc_proto::ibc::lightclients::tendermint::v1::ClientState as RawClientState;
 
 use crate::ics02_client::client_state::AnyClientState;
 use crate::ics02_client::client_type::ClientType;
-use crate::ics07_tendermint::error::{Error, Kind};
+use crate::ics02_client::trust_threshold::TrustThreshold;
+use crate::ics07_tendermint::error::Error;
 use crate::ics07_tendermint::header::Header;
 use crate::ics23_commitment::specs::ProofSpecs;
 use crate::ics24_host::identifier::ChainId;
@@ -56,32 +54,32 @@ impl ClientState {
     ) -> Result<ClientState, Error> {
         // Basic validation of trusting period and unbonding period: each should be non-zero.
         if trusting_period <= Duration::new(0, 0) {
-            return Err(Kind::InvalidTrustingPeriod
-                .context("ClientState trusting period must be greater than zero")
-                .into());
+            return Err(Error::invalid_trusting_period(
+                "ClientState trusting period must be greater than zero".to_string(),
+            ));
         }
         if unbonding_period <= Duration::new(0, 0) {
-            return Err(Kind::InvalidUnboundingPeriod
-                .context("ClientState unbonding period must be greater than zero")
-                .into());
+            return Err(Error::invalid_unbounding_period(
+                "ClientState unbonding period must be greater than zero".to_string(),
+            ));
         }
         if trusting_period >= unbonding_period {
-            return Err(Kind::InvalidUnboundingPeriod
-                .context("ClientState trusting period must be smaller than unbonding period")
-                .into());
+            return Err(Error::invalid_unbounding_period(
+                "ClientState trusting period must be smaller than unbonding period".to_string(),
+            ));
         }
 
         // Basic validation for the frozen_height parameter.
         if !frozen_height.is_zero() {
-            return Err(Kind::ValidationError
-                .context("ClientState cannot be frozen at creation time")
-                .into());
+            return Err(Error::validation(
+                "ClientState cannot be frozen at creation time".to_string(),
+            ));
         }
         // Basic validation for the latest_height parameter.
         if latest_height <= Height::zero() {
-            return Err(Kind::ValidationError
-                .context("ClientState latest height cannot be smaller or equal than zero")
-                .into());
+            return Err(Error::validation(
+                "ClientState latest height cannot be smaller or equal than zero".to_string(),
+            ));
         }
 
         Ok(Self {
@@ -111,11 +109,11 @@ impl ClientState {
         }
     }
 
-    /// Helper function to verify the upgrade client procedure.
+    /// Helper function for the upgrade chain & client procedures.
     /// Resets all fields except the blockchain-specific ones.
     pub fn zero_custom_fields(mut client_state: Self) -> Self {
         client_state.trusting_period = ZERO_DURATION;
-        client_state.trust_level = TrustThresholdFraction::default();
+        client_state.trust_level = TrustThreshold::ZERO;
         client_state.allow_update.after_expiry = false;
         client_state.allow_update.after_misbehaviour = false;
         client_state.frozen_height = Height::zero();
@@ -165,38 +163,37 @@ impl TryFrom<RawClientState> for ClientState {
         let trust_level = raw
             .trust_level
             .clone()
-            .ok_or_else(|| Kind::InvalidRawClientState.context("missing trusting period"))?;
+            .ok_or_else(Error::missing_trusting_period)?;
 
         Ok(Self {
             chain_id: ChainId::from_str(raw.chain_id.as_str())
-                .map_err(|_| Kind::InvalidRawClientState.context("Invalid chain identifier"))?,
-            trust_level: TrustThreshold::new(trust_level.numerator, trust_level.denominator)
-                .map_err(|e| Kind::InvalidTrustThreshold.context(e))?,
+                .map_err(Error::invalid_chain_identifier)?,
+            trust_level: trust_level
+                .try_into()
+                .map_err(|e| Error::invalid_trust_threshold(format!("{}", e)))?,
             trusting_period: raw
                 .trusting_period
-                .ok_or_else(|| Kind::InvalidRawClientState.context("missing trusting period"))?
+                .ok_or_else(Error::missing_trusting_period)?
                 .try_into()
-                .map_err(|_| Kind::InvalidRawClientState.context("negative trusting period"))?,
+                .map_err(|_| Error::negative_trusting_period())?,
             unbonding_period: raw
                 .unbonding_period
-                .ok_or_else(|| Kind::InvalidRawClientState.context("missing unbonding period"))?
+                .ok_or_else(Error::missing_unbonding_period)?
                 .try_into()
-                .map_err(|_| Kind::InvalidRawClientState.context("negative unbonding period"))?,
+                .map_err(|_| Error::negative_unbonding_period())?,
             max_clock_drift: raw
                 .max_clock_drift
-                .ok_or_else(|| Kind::InvalidRawClientState.context("missing max clock drift"))?
+                .ok_or_else(Error::missing_max_clock_drift)?
                 .try_into()
-                .map_err(|_| Kind::InvalidRawClientState.context("negative max clock drift"))?,
+                .map_err(|_| Error::negative_max_clock_drift())?,
             latest_height: raw
                 .latest_height
-                .ok_or_else(|| Kind::InvalidRawClientState.context("missing latest height"))?
-                .try_into()
-                .map_err(|_| Kind::InvalidRawHeight)?,
+                .ok_or_else(Error::missing_latest_height)?
+                .into(),
             frozen_height: raw
                 .frozen_height
-                .ok_or_else(|| Kind::InvalidRawClientState.context("missing frozen height"))?
-                .try_into()
-                .map_err(|_| Kind::InvalidRawHeight)?,
+                .ok_or_else(Error::missing_frozen_height)?
+                .into(),
             upgrade_path: raw.upgrade_path,
             allow_update: AllowUpdate {
                 after_expiry: raw.allow_update_after_expiry,
@@ -210,18 +207,15 @@ impl From<ClientState> for RawClientState {
     fn from(value: ClientState) -> Self {
         RawClientState {
             chain_id: value.chain_id.to_string(),
-            trust_level: Some(Fraction {
-                numerator: value.trust_level.numerator(),
-                denominator: value.trust_level.denominator(),
-            }),
+            trust_level: Some(value.trust_level.into()),
             trusting_period: Some(value.trusting_period.into()),
             unbonding_period: Some(value.unbonding_period.into()),
             max_clock_drift: Some(value.max_clock_drift.into()),
             frozen_height: Some(value.frozen_height.into()),
             latest_height: Some(value.latest_height.into()),
             proof_specs: ProofSpecs::cosmos().into(),
-            allow_update_after_expiry: false,
-            allow_update_after_misbehaviour: false,
+            allow_update_after_expiry: value.allow_update.after_expiry,
+            allow_update_after_misbehaviour: value.allow_update.after_misbehaviour,
             upgrade_path: value.upgrade_path,
         }
     }
@@ -232,9 +226,9 @@ mod tests {
     use std::time::Duration;
     use test_env_log::test;
 
-    use tendermint::trust_threshold::TrustThresholdFraction as TrustThreshold;
     use tendermint_rpc::endpoint::abci_query::AbciQuery;
 
+    use crate::ics02_client::trust_threshold::TrustThreshold;
     use crate::ics07_tendermint::client_state::{AllowUpdate, ClientState};
     use crate::ics24_host::identifier::ChainId;
     use crate::test::test_serialization_roundtrip;
