@@ -1,3 +1,4 @@
+use std::convert::Infallible;
 use std::convert::TryFrom;
 use std::time::SystemTime;
 
@@ -11,7 +12,7 @@ use ibc_proto::ibc::lightclients::tendermint::v1::ConsensusState as RawConsensus
 
 use crate::ics02_client::client_consensus::AnyConsensusState;
 use crate::ics02_client::client_type::ClientType;
-use crate::ics07_tendermint::error::{Error, Kind};
+use crate::ics07_tendermint::error::Error;
 use crate::ics07_tendermint::header::Header;
 use crate::ics23_commitment::commitment::CommitmentRoot;
 
@@ -33,6 +34,8 @@ impl ConsensusState {
 }
 
 impl crate::ics02_client::client_consensus::ConsensusState for ConsensusState {
+    type Error = Infallible;
+
     fn client_type(&self) -> ClientType {
         ClientType::Tendermint
     }
@@ -41,7 +44,7 @@ impl crate::ics02_client::client_consensus::ConsensusState for ConsensusState {
         &self.root
     }
 
-    fn validate_basic(&self) -> Result<(), Box<dyn std::error::Error>> {
+    fn validate_basic(&self) -> Result<(), Infallible> {
         unimplemented!()
     }
 
@@ -58,19 +61,21 @@ impl TryFrom<RawConsensusState> for ConsensusState {
     fn try_from(raw: RawConsensusState) -> Result<Self, Self::Error> {
         let proto_timestamp = raw
             .timestamp
-            .ok_or_else(|| Kind::InvalidRawConsensusState.context("missing timestamp"))?;
+            .ok_or_else(|| Error::invalid_raw_consensus_state("missing timestamp".into()))?;
 
         Ok(Self {
             root: raw
                 .root
-                .ok_or_else(|| Kind::InvalidRawConsensusState.context("missing commitment root"))?
+                .ok_or_else(|| {
+                    Error::invalid_raw_consensus_state("missing commitment root".into())
+                })?
                 .hash
                 .into(),
             timestamp: Utc
                 .timestamp(proto_timestamp.seconds, proto_timestamp.nanos as u32)
                 .into(),
             next_validators_hash: Hash::from_bytes(Algorithm::Sha256, &raw.next_validators_hash)
-                .map_err(|e| Kind::InvalidRawConsensusState.context(e.to_string()))?,
+                .map_err(|e| Error::invalid_raw_consensus_state(e.to_string()))?,
         })
     }
 }
